% VERSION 2.0, MARCH 1997, COPYRIGHT H. UHLIG.
% IMPRESP.M calculates and plots impulse responses to shocks one percent
% in size to each one of the exogenous variables z(t), keeping all
% others at zero in the first period.  It is assumed,
% that SOLVE.M has been executed before, so that the matrices
% NN, PP, QQ, RR and SS are available, describing the law of motion
%   x(t) = PP x(t-1) + QQ z(t)
%   y(t) = RR x(t-1) + SS z(t)
%   z(t) = NN z(t-1) + epsilon(t)
% The following options should have been chosen beforehand (see also file OPTIONS.M)
%   HORIZON    : how far out should the impulse responses be calculated
%   PERIOD     : number of periods per year, i.e. 12 for monthly, 4 for quarterly
%   DO_PLOTS   : = 1, if plots should be made, = 0, if not.
%   IMP_SELECT : a vector containing the indices of the variables to be
%             plotted.  To plot all variables, set IMP_SELECT = 1 : (m+n+k),
%             where m=dim(x), n=dim(y) and k=dim(z);
%   VARNAMES   : an array with (m+n+k) rows, containing the variable names.
%   The following options are set in OPTIONS.M:
%   TXT_MARKER : a number indicating where the label for the responses should be written.
%   DO_ENLARGE : = 1, if you want large font sizes for the text on your plots.  Good for slides.
%   PRINT_FIG  : = 1, if you want plots to be printed on your printer
%   SAVE_FIG   : = 1, if you want plots to be saved as encaps. postscript.  Set PRINT_FIG = 0 also.
% The program calculates:
% Response: the response of all variables x(t), y(t), z(t) to each shock.
%   Response is of size (m_states+n_endog+k_exog)*HORIZON.
%   Since Response is overwritten, each time a new shock is analyzed, 
%   the results are collected in 
% Resp_mat = [ Response to first shock
%              Response to second shock
%              ...                     ]
%
% The program also defines 
% Time_axis,m_states,n_endog,k_exog,II_contemp,II_lag,hndl,
% thus overwriting variables with these names that might have been used before.


% Copyright: H. Uhlig.  Feel free to copy, modify and use at your own risk.
% However, you are not allowed to sell this software or otherwise impinge
% on its free distribution.

% Most options are now set in OPTIONS.M

% Calculations
[m_states,k_exog] = size(QQ);
[n_endog,k_exog]  = size(SS);
Time_axis = (0 : (HORIZON-1))/PERIOD;
Resp_mat = [];
resp_mat1 = [];
for shock_counter = 1 : k_exog,
   Response = zeros(m_states+n_endog+k_exog,HORIZON);
   Response(m_states+n_endog+shock_counter,1) = 1;
   II_lag = [ PP, zeros(m_states,n_endog),zeros(m_states,k_exog)
              RR, zeros(n_endog, n_endog),zeros(n_endog, k_exog)
              zeros(k_exog,(m_states+n_endog)), NN                ];
   II_contemp = eye(m_states+n_endog+k_exog) + ...
        [ zeros(m_states,(m_states+n_endog)), QQ
          zeros(n_endog, (m_states+n_endog)), SS
          zeros(k_exog,  (m_states+n_endog)), zeros(k_exog,k_exog) ];
   % describing [x(t)',y(t)',z(t)']'= II_contemp*II_lag*[x(t-1)',y(t-1)',z(t-1)']';
   Response(:,1) = II_contemp*Response(:,1);
   for time_counter = 2 : HORIZON,
      Response(:,time_counter) = II_contemp*II_lag*Response(:,time_counter-1);
   end;
     % Response
if goo == 1
   gbs1=Response(7,:);
   gbs2=Response(16,:);
   gbs3=Response(8,:);
   gbs4=Response(15,:);
else; 
gbs1=Response(3,:);
   gbs2=Response(7,:);
   gbs3=Response(6,:);
   gbs4=Response(1,:);
end ;  
   gu=[gbs1', gbs2', gbs3', gbs4'];
   resp_mat1= [ resp_mat1, gu]; 
 
   Resp_mat = [ Resp_mat 
                Response ];
   if DO_PLOTS,
      hndl = plot(Time_axis,0*Time_axis, ...
                  Time_axis,Response(IMP_SELECT,:));
      set(hndl(2:max(size(hndl))),'LineWidth',2);
      grid;
      title(['Impulse responses to a shock in ', ...
          VARNAMES(m_states+n_endog+shock_counter,:)]);
      xlabel('Years after shock');
      ylabel('Percent deviation from steady state');
      for varindex = IMP_SELECT,
         text(Time_axis(min([TXT_MARKER,HORIZON/2])), ...
            Response(varindex,min([TXT_MARKER,HORIZON/2])),...
            VARNAMES(varindex,:));
      end;
      if DO_ENLARGE,
         enlarge;
      end;
      if PRINT_FIG,
         disp(['IMPRESP.M: Printing response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:),'...']);
         print;
      elseif SAVE_FIG
         if shock_counter == 1,
            disp(['IMPRESP.M: Saving response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:)]);
            disp( '         as encapsulated postscript file. Filename is impresp1.eps ...'); 
            print -deps impresp1.eps
         elseif shock_counter == 2,
            disp(['IMPRESP.M: Saving response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:)]);
            disp( '         as encapsulated postscript file. Filename is impresp2.eps ...'); 
            print -deps impresp2.eps
         elseif shock_counter == 3,
            disp(['IMPRESP.M: Saving response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:)]);
            disp( '         as encapsulated postscript file. Filename is impresp3.eps ...'); 
            print -deps impresp3.eps
         elseif shock_counter == 4,
            disp(['IMPRESP.M: Saving response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:)]);
            disp( '         as encapsulated postscript file. Filename is impresp4.eps ...'); 
            print -deps impresp4.eps
         elseif shock_counter == 5,
            disp(['IMPRESP.M: Saving response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:)]);
            disp( '         as encapsulated postscript file. Filename is impresp5.eps ...'); 
            print -deps impresp5.eps
         elseif shock_counter == 6,
            disp(['IMPRESP.M: Saving response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:)]);
            disp( '         as encapsulated postscript file. Filename is impresp6.eps ...'); 
            print -deps impresp6.eps
         elseif shock_counter == 7,
            disp(['IMPRESP.M: Saving response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:)]);
            disp( '         as encapsulated postscript file. Filename is impresp7.eps ...'); 
            print -deps impresp7.eps
         elseif shock_counter == 8,
            disp(['IMPRESP.M: Saving response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:)]);
            disp( '         as encapsulated postscript file. Filename is impresp8.eps ...'); 
            print -deps impresp8.eps
         elseif shock_counter == 9,
            disp(['IMPRESP.M: Saving response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:)]);
            disp( '         as encapsulated postscript file. Filename is impresp9.eps ...'); 
            print -deps impresp9.eps
         elseif shock_counter > 9,
            disp('IMPRESP.M: Sorry!  I can only save nine of your impulse responses!');
            disp(['         So, response to a shock in ',VARNAMES(m_states+n_endog+shock_counter,:),' is not saved.']);
            disp('         If you want me to save more, you need to change my code.');
         end;
      else                     
         disp('Inspect figure. Hit key when ready...');
         pause;
      end;
   end;
end;

% variance decomposition of a system with y,m,r, pi 

%i3 = resp_mat1(:,1);
%i5 = resp_mat1(:,2);
%i6 = resp_mat1(:,3);
%i7 = resp_mat1(:,4); 
 
%i13 = resp_mat1(:,5);
%i15 = resp_mat1(:,6);
%i16 = resp_mat1(:,7);
%i17 = resp_mat1(:,8); 
 
%i23 = resp_mat1(:,9);
%i25 = resp_mat1(:,10);
%i26 = resp_mat1(:,11);
%i27 = resp_mat1(:,12); 
 
%i33 = resp_mat1(:,13);
%i35 = resp_mat1(:,14);
%i36 = resp_mat1(:,15);
%i37 = resp_mat1(:,16); 

%x3(:,1) = i3;
%x5(:,1) = i5;
%%x6(:,1) = i6;
%x7(:,1) = i7; 
 
%x13(:,1) = i13;
%x15(:,1) = i15;
%x16(:,1) = i16;
%x17(:,1) = i17; 
 
%x23(:,1) = i23;
%x25(:,1) = i25;
%x26(:,1) = i26;
%x27(:,1) = i27; 
 
%x33(:,1) = i33;
%x35(:,1) = i35;
%x36(:,1) = i36;
%x37(:,1) = i37; 

 
%xi3=0;
%xi5=0;
%xi6=0;
%xi7=0;
%xi13=0;
%xi15=0;
%%xi16=0;
%xi17=0;
%xi23=0;
%xi25=0;
%xi26=0;
%xi27=0;
%xi33=0;
%xi35=0;
%xi36=0;
%xi37=0;

%for i=1:40

%%si3(i,1) = x3(i,1)^2; 
%si5(i,1) = x5(i,1)^2; 
%si6(i,1) = x6(i,1)^2; 
%si7(i,1) = x7(i,1)^2; 
 
%si13(i,1) = x13(i,1)^2; 
%si15(i,1) = x15(i,1)^2; 
%si16(i,1) = x16(i,1)^2; 
%si17(i,1) = x17(i,1)^2; 
 
%si23(i,1) = x23(i,1)^2; 
%si25(i,1) = x25(i,1)^2; 
%si26(i,1) = x26(i,1)^2; 
%si27(i,1) = x27(i,1)^2; 
 
%si33(i,1) = x33(i,1)^2; 
%si35(i,1) = x35(i,1)^2; 
%si36(i,1) = x36(i,1)^2; 
%si37(i,1) = x37(i,1)^2; 

%xi3 = xi3+si3(i,1);
%xi5 = xi5+si5(i,1);
%xi6 = xi6+si6(i,1);
%xi7 = xi7+si7(i,1);

%xi13 = xi13+si13(i,1);
%xi15 = xi15+si15(i,1);
%xi16 = xi16+si16(i,1);
%xi17 = xi17+si17(i,1);

%%xi23 = xi23+si23(i,1);
%xi25 = xi25+si25(i,1);
%xi26 = xi26+si26(i,1);
%xi27 = xi27+si27(i,1);

%xi33 = xi33+si33(i,1);
%xi35 = xi35+si35(i,1);
%xi36 = xi36+si36(i,1);
%xi37 = xi37+si37(i,1);

%end

%ix1 = xi3+xi13+xi23+xi33;
%ix2 = xi5+xi15+xi25+xi35;
%ix3 = xi6+xi16+xi26+xi36;
%ix4 = xi7+xi17+xi27+xi37;

%vr(1,1)= xi3/ix1;
%%vr(1,2)= xi13/ix1;
%vr(1,3)= xi23/ix1;
%vr(1,4)= xi33/ix1;
%vr(2,1)= xi5/ix2;
%vr(2,2)= xi15/ix2;
%vr(2,3)= xi25/ix2;
%vr(2,4)= xi35/ix2;
%vr(3,1)= xi6/ix3;
%vr(3,2)= xi16/ix3;
%vr(3,3)= xi26/ix3;
%vr(3,4)= xi36/ix3;
%vr(4,1)= xi7/ix4;
%vr(4,2)= xi17/ix4;
%vr(4,3)= xi27/ix4;
%vr(4,4)= xi37/ix4;
