% NK1.m ,  February 15, 2005, version 2.3
% Solves a sticky price model with 3 equations in 3 inflation, the gap and interest rates.

% the (log-linear) model is:
% y(t) = E_{t} y_{t+1} + 1/ phi (i(t)-E_t(inf_{t+1})-1) + e1(t)
% inf(t) = beta E_t (inf(t+1) + (phi*(1-zeta_p beta)(1-zeta_p))/(zeta_p) y_t + e2(t)
% i(t)       = phi_r i_(t-1)+ (1-phi_r)(phi_p inf(t-1) +phi_y y(t-1)) + e3(t)

% Given that the Taylor rule is backward looking we need to defined 
% auxiliary variables  for lagged inflation and output
 

%%%%%%%%%%%%%%%%%%%% EXERCISES FOR PRACTICE %%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%% easy %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  exercise 1: set phi_P < 1 and see what happens
%  exercise 2: reduce price stickyness to 0.25
%  exercise 3: set the persistence of the shocks to zero

%%%%%%%%%%%%%%%%%%%%%%% more difficult ones %%%%%%%%%%%%%%%%%%%%%%%%%%%%
% exercise 4: allow for external habit persistence in consumption. derive impulse
%             responses in this case and compare them with the baseline
% NOTE:       with external habit, the first equation is
% y(t) = h/(1+h) y_{t-1} + 1/(1+h) E_{t} y_{t+1} + (1-h)/[(1+h)phi] (i(t)-E_t(inf_{t+1})-1) + e1(t)
% exercise 5: consider a policy rule where the interest rate depends on
%             current values of output and inflation. Compare impulse 
%             responses with those in the baseline case.
%  NOTE:      the third equation is  
% i(t)       = phi_r i_(t-1)+ (1-phi_r)(phi_p inf(t) +phi_y y(t)) + e3(t)
% exercise 6: compute variability and persistence (AR1 coefficient) of inflation  
%             and compare them  with those in the data.
% exercise 7: allow for indexation in prices. In this case the second
%            equation becomes
% inf(t) = omega/ (1+omega beta) inf(t-1) + E_t beta/(1+omega beta) inf(t+1) +
%      (phi*(1-zeta_p beta)(1-zeta_p))/((1+omega beta)*zeta_p) y_t + e2(t)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

% Parameters:
% structural parameters:
beta = 1.04^(-1/4) ; % discount factor
phi = 2.0; % relative risk aversion coefficient 
zeta_p = 0.75; % degree of price stickiness
phi_r = 0.8; % interest rate smoothing parameter
phi_p = 1.5; % weight on inflation in interest rate rule 
phi_y = 0.2; % weight on output gap in interest rate rule
zeteps = (1-zeta_p)*(1-beta*zeta_p)/zeta_p;
% auxiliary parameters
rhoe1 = 0.8; % persistence of the shocks 
rhoe2 = 0.8; 
rhoe3 = 0.0;
sigma_e1 = 1.0; % variance of the shocks
sigma_e2 = 1.0; 
sigma_e3 = 1.0;

% order of the variables in the program 
VARNAMES = ['interest rate ',
            'inflation     ',
            'output gap    ',
            'lag inflation ',
            'lag gap       ',
            'IS shock      ',
            'AS shock      ',
            'Taylor shock  '];


% Endogenous state variables "x(t)": interest rate, inflation, gap
% Endogenous other variables "y(t)": lagged inflation and lagged gap
% Exogenous state variables  "z(t)": 3 equation shocks

% Build matrices for the format
% 0 = AA x(t) + BB x(t-1) + CC y(t) + DD z(t)
% 0 = E_t [ FF x(t+1) + GG x(t) + HH x(t-1) + JJ y(t+1) + KK y(t) + LL z(t+1) + MM z(t)]
% z(t+1) = NN z(t) + epsilon(t+1) with E_t [ epsilon(t+1) ] = 0,

% NOTE: the first equation is the Taylor rule. The next two are identities

%     i(t),  pi(t),   x(t):
AA = [ -1,    0,         0
        0,    0,         0
        0,    0,         0]; 

%      i(t-1), pi(t-1),   x(t-1):
BB = [ phi_r,   0,       0
          0,    1,       0
          0,    0,       1]; 

%           lag-pi(t-1),   lag-x(t-1)
CC = [    (1-phi_r)*phi_p, (1-phi_r)*phi_y
                       -1,      0
                        0,     -1];

%     e1(t)   e2(t) e3(t) 
DD = [  0,     0,    1
        0,    0,    0
        0,    0,    0]; 


% NOTE the first equation is the euler equation, the second the Phillips curve    
    
%    i(t+1),  pi(t+1),   x(t+1):
FF = [  0,    1/phi,       1
        0,    beta,        0];


%      i(t),     pi(t),   x(t):
GG = [ -1/phi,    0,        -1
            0,    -1,     phi*zeteps];
        
%     i(t-1),     pi(t-1),   x(t-1):
HH = [ 0,         0,        0
       0,         0,        0];

%  lag-pi(t+1), lag-x(t+1)
JJ = [   0,      0
         0,      0];

% lag-pi(t), lag-x(t)
KK = [ 0,      0
       0,      0];

%     e1(t+1)   e2(t+1) e3(t+1) 
LL = [ 0,  0,  0
       0,  0,  0];

%     e1(t)   e2(t) e3(t) 
MM = [ 1,  0,  0
       0,  1,  0];

% NOTE: these are the processes for the shocks
   
NN = zeros(3,3);
NN(1,1) = rhoe1;
NN(2,2) = rhoe2;
NN(3,3) = rhoe3;

Sigma = [ sigma_e1^2,          0,        0
                   0, sigma_e2^2,        0
                   0,          0, sigma_e3^2];

               
               
% solving the model 

DISPLAY_IMMEDIATELY = 1;
MANUAL_ROOTS = 0;
warnings = [];
IGNORE_VV_SING = 1;


TOL = 0.000000001;

[l_equ,m_states] = size(AA);
[l_equ,n_endog ] = size(CC);
k_exog = min(size(NN));
sumimag = sum(sum(abs(imag(AA))))+sum(sum(abs(imag(BB))))+sum(sum(abs(imag(CC))));
sumabs  = sum(sum(abs(AA)))      +sum(sum(abs(BB)))      +sum(sum(abs(CC)));



if sumimag / sumabs > .000001,
  message = ['SOLVE.M: I hate to point this out to you, but some of your matrices    '  
             '         contain complex numbers, which does not make much sense. You  '
             '         should check your steady state parameters and calculations.   '
             '         I will proceed anyhow, but you will probably get nonsense.    '];
  if DISPLAY_IMMEDIATELY, disp(message); end;
  warnings = [warnings;message];
end;

if rank(CC)<n_endog,
  message = 'SOLVE.M: Sorry!  Rank(CC) needs to be at least n! Cannot solve for PP. ';
  if DISPLAY_IMMEDIATELY, disp(message); end;
  warnings = [warnings;message];
else
  CC_plus = pinv(CC);
  CC_0 = (null(CC'))';
  Psi_mat   = [ zeros(l_equ-n_endog,m_states)
              FF - JJ*CC_plus*AA           ];
  Gamma_mat = [ CC_0 * AA
                JJ*CC_plus*BB - GG + KK*CC_plus*AA ];
  Theta_mat = [ CC_0 * BB
                KK*CC_plus*BB - HH                 ];
  Xi_mat    = [ Gamma_mat,     Theta_mat
                eye(m_states), zeros(m_states) ];
  Delta_mat = [ Psi_mat,       zeros(m_states)
                zeros(m_states), eye(m_states) ];
  [Xi_eigvec,Xi_eigval] = eig(Xi_mat,Delta_mat);
  if rank(Xi_eigvec)<m_states,
     message = 'SOLVE.M: Sorry! Xi is not diagonalizable! Cannot solve for PP.         ';
     if DISPLAY_IMMEDIATELY, disp(message);  end;
     warnings = [warnings;message];
  else
    [Xi_sortabs,Xi_sortindex] = sort(abs(diag(Xi_eigval)));
    Xi_sortvec = Xi_eigvec(1:2*m_states,Xi_sortindex);
    Xi_sortval = diag(Xi_eigval(Xi_sortindex,Xi_sortindex));
    Xi_select = 1 : m_states;
    if imag(Xi_sortval(m_states))~=0,
      if (abs( Xi_sortval(m_states) - conj(Xi_sortval(m_states+1)) ) < TOL),
      % NOTE: THIS LAST LINE MIGHT CREATE PROBLEMS, IF THIS EIGENVALUE OCCURS MORE THAN ONCE!!
      % IF YOU HAVE THAT PROBLEM, PLEASE TRY MANUAL ROOT SELECTION.  
        drop_index = 1;
        while (abs(imag(Xi_sortval(drop_index)))>TOL) & (drop_index < m_states),
          drop_index = drop_index + 1;
        end;
        if drop_index >= m_states,
          message = ['SOLVE.M: You are in trouble. You have complex eigenvalues, and I cannot'
                     '   find a real eigenvalue to drop to only have conjugate-complex pairs.'
                     '   Put differently: your PP matrix will contain complex numbers. Sorry!'];
          if DISPLAY_IMMEDIATELY, disp(message);  end;
          warnings = [warnings;message];
          if m_states == 1,
            message = ['   TRY INCREASING THE DIMENSION OF YOUR STATE SPACE BY ONE!            '
                       '   WATCH SUNSPOTS!                                                     '];                     
            if DISPLAY_IMMEDIATELY, disp(message); end;
            warnings = [warnings;message];
            end;
        else
          message = ['SOLVE.M: I will drop the lowest real eigenvalue to get real PP.        '
                     '         I hope that is ok. You may have sunspots.                     ']; 
          if DISPLAY_IMMEDIATELY, disp(message);  end;
          warnings = [warnings;message];
          Xi_select = [ 1: (drop_index-1), (drop_index+1):(m_states+1)];
       end;
    end;
  end;
    if MANUAL_ROOTS,
      message = ['SOLVE.M: You have chosen to select roots manually.  I am crossing my   '
                 '         fingers that you are doing it correctly.  In particular,      '
                 '         you should have defined Xi_manual.  Type help solve           '
                 '         and inspect SOLVE.M to get further information on how to do it'];
      if DISPLAY_IMMEDIATELY, disp(message);  end;
      warnings = [warnings;message];
      if exist('Xi_manual'),
         Xi_select = Xi_manual;
      else
         message = ['SOLVE.M: You have not defined Xi_manual.  Either define it or turn off '
                    '         the manual roots selection procedure with                     '
                    '         MANUAL_ROOTS = 0                                              '
                    '         Right now, I better let your calculations crash - sorry!      '
                    '         If you get results, they are based on previous calculations.  '];
         disp(message);
         warnings = [warnings;message];
    end;
    else
      if max(Xi_select) < 2*m_states,
        if Xi_sortabs(max(Xi_select)+1) < 1 - TOL,
          message = ['SOLVE.M: You may be in trouble. There are stable roots NOT used for PP.'
                     '         I have used the smallest roots: I hope that is ok.            '  
                     '         If not, try manually selecting your favourite roots.          '
                     '         For manual root selection, take a look at the file solve.m    '
                     '         Watch out for sunspot solutions.                              '];
          if DISPLAY_IMMEDIATELY, disp(message);  end;
          warnings = [warnings;message];
      end; 
    end;
    if max(abs(Xi_sortval(Xi_select)))  > 1 + TOL,
      message = ['SOLVE.M: You may be in trouble.  There are unstable roots used for PP. '
                 '         Keep your fingers crossed or change your model.               '];
      if DISPLAY_IMMEDIATELY, disp(message);  end;
      warnings = [warnings;message];
   end;
    if abs( max(abs(Xi_sortval(Xi_select))) - 1  ) < TOL,
      message = ['SOLVE.M: Your matrix PP contains a unit root. You probably do not have '
                 '         a unique steady state, do you?  Should not be a problem, but  '
                 '         you do not have convergence back to steady state after a shock'
                 '         and you should better not trust long simulations.             '];
      if DISPLAY_IMMEDIATELY, disp(message);  end;
      warnings = [warnings;message];
  end;

    Lambda_mat = diag(Xi_sortval(Xi_select));
    Omega_mat  = [Xi_sortvec((m_states+1):(2*m_states),Xi_select)];
    if rank(Omega_mat)<m_states,
      message = 'SOLVE.M: Sorry! Omega is not invertible. Cannot solve for PP.          ';
      if DISPLAY_IMMEDIATELY, disp(message);   end;
      warnings = [warnings;message];
    else
      PP = Omega_mat*Lambda_mat/Omega_mat;
      PP_imag = imag(PP);
      PP = real(PP);
      if sum(sum(abs(PP_imag))) / sum(sum(abs(PP))) > .000001,
        message = ['SOLVE.M: PP is complex.  I proceed with the real part only.            '  
                   '         Hope that is ok, but you are probably really in trouble!!     '
                   '         You should better check everything carefully and be           '
                   '         distrustful of all results which follow now.                  '];
        if DISPLAY_IMMEDIATELY, disp(message);  end;
        warnings = [warnings;message];
    end;
      RR = - CC_plus*(AA*PP+BB);
      VV = [ kron(eye(k_exog),AA),   kron(eye(k_exog),CC)
             kron(NN',FF)+kron(eye(k_exog),(FF*PP+JJ*RR+GG)), ...
                                     kron(NN',JJ)+kron(eye(k_exog),KK) ];
      if ( (rank(VV) < k_exog*(m_states+n_endog)) & ...
                                       (~IGNORE_VV_SING) ),
        message = ['SOLVE.M: Sorry! V is not invertible.  Cannot solve for QQ and SS. You  '
                   '         can try setting IGNORE_VV_SING = 1 and wish for the best...   '];
        if DISPLAY_IMMEDIATELY, disp(message); end;
        warnings = [warnings;message];
      else
        if ( (rank(VV) < k_exog*(m_states+n_endog)) ),
          message = ['SOLVE.M: Warning! V is not invertible.  However, you have set          '
                     '         IGNORE_VV_SING = 1, and thus, since you have told me to       '
                     '         ignore this, I will proceed.  Keep your fingers crossed...    ']
          if DISPLAY_IMMEDIATELY, disp(message);   end;
          warnings = [warnings;message];
    end;
        LLNN_plus_MM = LL*NN + MM;
        QQSS_vec = - VV \ [ DD(:)
                            LLNN_plus_MM(:) ];
        if max(abs(QQSS_vec)) == Inf,
           message = ['SOLVE.M: You probably are in trouble!  QQ or SS contain undefined      '
                      '         entries! Most likely, the matrix VV is not invertible.        '];
           if DISPLAY_IMMEDIATELY, disp(message);  end;
           warnings = [warnings;message];
    end;           
        QQ = reshape(QQSS_vec(1:m_states*k_exog),m_states,k_exog);
        SS = reshape(QQSS_vec((m_states*k_exog+1):((m_states+n_endog)*k_exog)),n_endog,k_exog);
        WW = [ eye(m_states)         , zeros(m_states,k_exog)
               RR*pinv(PP)           , (SS-RR*pinv(PP)*QQ) 
               zeros(k_exog,m_states), eye(k_exog)            ];
        end;
      end;
    end;
  end;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% THE solution is of the form
% x(t) = PP x(t-1)+ QQ z(t)
% y(t)= RR x(t-1)+ SS z(t)
% z(t) = NN z(t-1)+ e(t) 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% if you want to see the law of motion use these
%disp('PP: Recursive equilibrium law of motion for x(t) on x(t-1):');
%disp(PP);
%disp('QQ: Recursive equilibrium law of motion for x(t) on z(t):');
%disp(QQ);
%disp(' ');
%disp('RR: Recursive equilibrium law of motion for y(t) on x(t-1):');
%disp(RR);
%disp('SS: Recursive equilibrium law of motion for y(t) on z(t):');
%disp(SS);
%disp(' ');


% *****************
% impulse responses
% *****************
horizon = 20;
period  = 4;
nvar = 3;

[m_states,k_exog] = size(QQ);
[n_endog,k_exog]  = size(SS);
Time_axis = (0 : (horizon-1))/period;
Resp_mat = [];

% *********************************************************************
% 1 = is shock; 2 = phillips curve shock; 3 = monetary policy shock
% *********************************************************************


for i = 1 : nvar,
shock_counter = i;

Response = zeros((m_states+n_endog+k_exog),horizon);
Response(m_states+n_endog+shock_counter,1) = 1;

   
II_lag = [ PP, zeros(m_states,n_endog),zeros(m_states,k_exog)
           RR, zeros(n_endog, n_endog),zeros(n_endog, k_exog)
           zeros(k_exog,(m_states+n_endog)), NN                ];
              
II_contemp = eye(m_states+n_endog+k_exog) + ...
        [ zeros(m_states,(m_states+n_endog)), -QQ
          zeros(n_endog, (m_states+n_endog)), -SS
          zeros(k_exog,  (m_states+n_endog)), zeros(k_exog,k_exog) ];
       
Response(:,1) = inv(II_contemp)*Response(:,1);
       
for time_counter = 2 : horizon,
     Response(:,time_counter) = inv(II_contemp)*II_lag*Response(:,time_counter-1);
end;
      
response_output_gap(i,:)    = Response(3,:);
response_infl(i,:)          = Response(2,:);
response_interest_rate(i,:) = Response(1,:);
 
end;

xx=[response_output_gap(1,:)' response_infl(1,:)' response_interest_rate(1,:)' 
    response_output_gap(2,:)' response_infl(2,:)' response_interest_rate(2,:)'
    response_output_gap(3,:)' response_infl(3,:)' response_interest_rate(3,:)'];

%graphing impulse responses
figure
subplot(3,3,1);
plot(response_output_gap(1,:),'-');
title('IS shocks');
ylabel('Output gap');
subplot(3,3,4);
plot(response_infl(1,:),'-');
ylabel('inflation');
subplot(3,3,7);
plot(response_interest_rate(1,:),'-');
ylabel('interest rate');
subplot(3,3,2);
plot(response_output_gap(2,:),'-');
title('Phillips shocks');
subplot(3,3,5);
plot(response_infl(2,:),'-');
subplot(3,3,8);
plot(response_interest_rate(2,:),'-');
subplot(3,3,3);
plot(response_output_gap(3,:),'-');
title('Monetary shocks');
subplot(3,3,6);
plot(response_infl(3,:),'-');
subplot(3,3,9);
plot(response_interest_rate(3,:),'-');

%saving simulated impulse responses
%diary calibration3a.dat
%xx
%diary off
 