% NK1_solution.m , February 15, 2005, version 1.2
% Solves a sticky price model of 3 equations in output gap, inflation, nominal rate

% possible to have i) habit persistence in consumption; ii) price indexation

% model is:
% y(t) = gam/(1+gam) y(t-1)+ 1/(1+gam) E_{t} y_{t+1} + 1/ phi (i(t)-E_t(Delta p_{t+1})) + e1(t)
% Delta p(t) = omega/ (1+omega beta) Delta p(t-1) + beta/(1+omega beta) (Delta p(t+1) +
%      ((phi+vartheta)*(1-zeta_p beta)(1-zeta_p))/((1+omega beta)*zeta_p) y_t + e2(t)
% i_t       = phi_r i_(t-1)+ (1-phi_r)(phi_p Delta p_t +phi_x y(t)) + e3(t)

% Given that the Taylor rule is backward looking we have to defined auxiliary variables 
% for lagged inflation and output. Solved with Uhlig's algorithm. 

clear all; close all;
clc;

% Setting parameters:

% economic parameters:
bet = 0.99 ; % discount factor
phi = 2.0; % relative risk aversion coefficient
zeta_p = 0.68; % degree of price stickiness
vartheta = 3.0; % elasticity of labor supply
phi_r = 0.6; % interest rate smoothing parameter
phi_p = 1.5; % weight on inflation in interest rate rule 
phi_x = 1.1; % weight on output gap in interest rate rule

% the two new parameters
omega = 0.4; %price indexation parameter
gam = 0.7 ; % habit persistence parameter


zetaps = (1-zeta_p)*(1-bet*zeta_p)/(zeta_p);

% auxiliary parameters
rho1 = 0.65; 
rho2 = 0.65; 
sigma_e1 = 0.01; 
sigma_e2 = 0.01; 
sigma_e3 = 0.01;

VARNAMES = ['interest rate ',
            'inflation     ',
            'output gap    ',
            'lag inflation ',
            'lag gap       ',
            'IS shock      ',
            'AS shock      ',
            'Taylor shock  '];

 
% Endogenous state variables "x(t)": interest rate, inflation, gap
% Endogenous other variables "y(t)": lagged inflation and lagged gap
% Exogenous state variables  "z(t)": 3 equation shocks

% Find matrices for format
% 0 = AA x(t) + BB x(t-1) + CC y(t) + DD z(t)
% 0 = E_t [ FF x(t+1) + GG x(t) + HH x(t-1) + JJ y(t+1) + KK y(t) + LL z(t+1) + MM z(t)]
% z(t+1) = NN z(t) + epsilon(t+1) with E_t [ epsilon(t+1) ] = 0,

% for i(t),  pi(t),   x(t):
AA = [ -1,    0,         0
        0,    0,         0
        0,    0,         0]; 

% for i(t-1), pi(t-1),   x(t-1):
BB = [ phi_r,   0,       0
          0,    1,       0
          0,    0,       1]; 

%Order:   lag-pi(t-1), lag-x(t-1)
CC = [    (1-phi_r)*phi_p, (1-phi_r)*phi_x
                       -1,      0
                        0,     -1];

%     e1(t)   e2(t) e3(t) 
DD = [  0,     0,    1
        0,    0,    0
        0,    0,    0]; 


% for i(t+1),  pi(t+1),            x(t+1):
FF = [  0,    1/phi,             1/(1+gam)
        0,    bet/(1+omega*bet),        0];


% for i(t),       pi(t),   x(t):
GG = [ -1/phi,    0,        -1
            0,    -1,     (phi+vartheta)*zetaps/(1+omega*bet)];
        
% for i(t-1),     pi(t-1),   x(t-1):

HH = [ 0,  0,  0
       0,  0,  0];
% for lag-pi(t+1), lag-x(t+1)

JJ = [ 0,  0
       0,  0];

   % for lag-pi(t), lag-x(t)

KK = [ 0,  gam/(1+gam)
    omega/(1+bet*omega),0];

%     e1(t+1)   e2(t+1) e3(t+1) 
LL = [ 0,  0,  0
       0,  0,  0];

%     e1(t)   e2(t) e3(t) 

MM = [ 1,  0,  0
       0,  1,  0];

NN = zeros(3,3);
NN(1,1) = rho1;
NN(2,2) = rho2;
NN(3,3) = 0;

Sigma = [ sigma_e1^2,          0,        0
                   0, sigma_e2^2,        0
                   0,          0, sigma_e3^2];

               
% solve the model
 
solving;


% *****************
% impulse responses
% *****************

horizon = 20;
period  = 4;
nvar = 3;

[m_states,k_exog] = size(QQ);
[n_endog,k_exog]  = size(SS);
Time_axis = (0 : (horizon-1))/period;
Resp_mat = [];

% *********************************************************************
% 1 = is shock; 2 = phillips curve shock; 3 = monetary policy shock
% *********************************************************************


for i = 1 : nvar,
shock_counter = i;

Response = zeros((m_states+n_endog+k_exog),horizon);
Response(m_states+n_endog+shock_counter,1) = 1;

   
II_lag = [ PP, zeros(m_states,n_endog),zeros(m_states,k_exog)
           RR, zeros(n_endog, n_endog),zeros(n_endog, k_exog)
           zeros(k_exog,(m_states+n_endog)), NN                ];
              
II_contemp = eye(m_states+n_endog+k_exog) + ...
        [ zeros(m_states,(m_states+n_endog)), -QQ
          zeros(n_endog, (m_states+n_endog)), -SS
          zeros(k_exog,  (m_states+n_endog)), zeros(k_exog,k_exog) ];
       
Response(:,1) = inv(II_contemp)*Response(:,1);
       
for time_counter = 2 : horizon,
     Response(:,time_counter) = inv(II_contemp)*II_lag*Response(:,time_counter-1);
end;
      
response_output_gap(i,:)    = Response(3,:);
response_infl(i,:)          = Response(2,:);
response_interest_rate(i,:) = Response(1,:);
 

% response of price level
%response_price = zeros(horizon,1);
%response_price(1,1) = response_infl(1);
%for i = 2:horizon
%	response_price(i,1) = response_price(i-1,1)+response_infl(i,1);
%end 

%zero_ax = zeros(horizon,1);
end;

xx=[response_output_gap(1,:)' response_infl(1,:)' response_interest_rate(1,:)' 
    response_output_gap(2,:)' response_infl(2,:)' response_interest_rate(2,:)'
    response_output_gap(3,:)' response_infl(3,:)' response_interest_rate(3,:)'];


par_value=[phi vartheta zeta_p phi_r phi_p phi_x bet omega gam rho1 rho2];
 
 
%graphing impulse responses
figure
subplot(3,3,1);
plot(response_output_gap(1,:),'-');
title('IS shocks');
ylabel('Output gap');
subplot(3,3,4);
plot(response_infl(1,:),'-');
ylabel('inflation');
subplot(3,3,7);
plot(response_interest_rate(1,:),'-');
ylabel('interest rate');
subplot(3,3,2);
plot(response_output_gap(2,:),'-');
title('Phillips shocks');
subplot(3,3,5);
plot(response_infl(2,:),'-');
subplot(3,3,8);
plot(response_interest_rate(2,:),'-');
subplot(3,3,3);
plot(response_output_gap(3,:),'-');
title('Monetary shocks');
subplot(3,3,6);
plot(response_infl(3,:),'-');
subplot(3,3,9);
plot(response_interest_rate(3,:),'-');



 
% simulating data out of the model
yy= zeros(8,600);
yy0=zeros(8,1);
 ee=zeros(8,600);
 ee(6:8,:)=randn(3,600);
 for tt=1:600;
    yy(:,tt) = II_lag *yy0 +II_contemp*ee(:,tt);
    yy0=yy(:,tt);
end 

yy1=zeros(3,500);
yy1(:)=yy(1:3,101:600)';

%saving simulated data
%diary simulation3c.dat
%yy1'
%diary off
