% RBC1.m: Basic RBC model with indivisible labor a-la Hansen (1985), version 2.1, 11-3-2005
% the derivations that led to the setup of this program are in the file rbc.tex

%%%%%%%%%%%%%%%%%%%%%%%Basic exercises %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% exercise 1: change the persistence of technology shocks and compare results
% exercise 2: set the depreciation rate of capital to 1 and compare results
% exercise 3: double the capital share and compare results
% 
%%%%%%%%%%%%%%%%%%%%%%% More difficult exercise %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% exercise 4: add habit persistence in the consumption (using external habit assumption), 
%             compare responses  and HP autocovariances and compare them with the original ones.
%  NOTE:    in this case equation 6 below becomes
% 6) 0 = E_t [ - h c(t-1)- eta c(t+1) + (1-h) r(t+1) + eta (1+h)c(t) ]
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%  Parameters:
N_bar     = 1.0/3;  % Steady state employment is a third of total time endowment
Z_bar     = 1; % Normalization
rho       = .36; % Capital share
delta     = .025; % Depreciation rate for capital
R_bar     = 1.01; % One percent real interest per quarter
eta       = 1.0; % constant of relative risk aversion = 1/(coeff. of intertemporal substitution)
psi       = .95; % autocorrelation of technology shock
sigma_eps = .712; % Standard deviation of technology shock.  Units: Percent.

% Steady state:
betta   = 1.0/R_bar;  % Discount factor beta
YK_bar  = (R_bar + delta - 1)/rho;  % = Y_bar / K_bar
K_bar   = (YK_bar / Z_bar)^(1.0/(rho-1)) * N_bar;
I_bar   = delta * K_bar;
Y_bar   = YK_bar * K_bar;
C_bar   = Y_bar - delta*K_bar;
A       =  C_bar^(-eta) * (1 - rho) * Y_bar/N_bar; % Parameter in utility function

 
% order of the variables in the model
VARNAMES = ['capital    ',
            'consumption',
            'output     ',
            'labor      ',
            'interest   ',
            'investment ',
            'technology '];

% LOG-LINEARIZED CONDITIONS  
%
% 1) 0 = - I i(t) - C c(t) + Y y(t)
% 2) 0 = I i(t) - K k(t) + (1-delta) K k(t-1)
% 3) 0 = rho k(t-1) - y(t) + (1-rho) n(t) + z(t)
% 4) 0 = -eta c(t) + y(t) - n(t)
% 5) 0 = - rho Y/K k(t-1) + rho Y/K y(t) - R r(t)
% 6) 0 = E_t [ - eta c(t+1) + r(t+1) + eta c(t) ]
% 7) z(t+1) = psi z(t) + epsilon(t+1)
% 7 equations, 7 variables.
%
% Endogenous state variables "x(t)": k(t)
% Endogenous other variables "y(t)": c(t), y(t), n(t), r(t), i(t)
% Exogenous state variables  "z(t)": z(t).

% Find matrices for format
% 0 = AA x(t) + BB x(t-1) + CC y(t) + DD z(t)
% 0 = E_t [ FF x(t+1) + GG x(t) + HH x(t-1) + JJ y(t+1) + KK y(t) + LL z(t+1) + MM z(t)]
% z(t+1) = NN z(t) + epsilon(t+1) with E_t [ epsilon(t+1) ] = 0,

%    k(t):
AA = [ 0
       - K_bar
       0
       0
       0 ];

%     k(t-1):
BB = [ 0
       (1-delta)*K_bar
       rho
       0
       - rho * YK_bar ];

%       consumption   output      labor     interest  investment
CC = [    -C_bar,      Y_bar,      0,        0,        -I_bar % Equ. 1)
          0,           0,          0,        0,        I_bar  % Equ. 2)
          0,           -1,         1-rho,    0,        0      % Equ. 3)      
          -eta,        1,          -1,       0,        0      % Equ. 4)
          0,           rho*YK_bar, 0,        - R_bar,  0 ];   % Equ. 5)

%       tech shock 
DD = [ 0
       0
       1
       0
       0 ];

FF = [ 0 ];

GG = [ 0 ];

HH = [ 0 ];

% equation 6)
JJ = [ -eta,  0,  0,  1,  0];

KK = [ eta,   0,  0,  0,  0];

LL = [ 0 ];

MM = [ 0 ];

% shocks process
NN = [psi];

Sigma = [ sigma_eps^2  ];

% Setting the options:
HORIZON    = 20; % how far out should the impulse responses be calculated
PERIOD     = 4;  % number of periods per year, i.e. 12 for monthly, 4 for quarterly

TXT_MARKER = 3; % Where to put the labels on the graph, a number between 1 and HORIZON/2.
DO_ENLARGE = 0; % =1, if you need large letter sizes for overheads, say.
PRINT_FIG  = 0; % Set = 1, if you want to sent figures directly to printer
SAVE_FIG   = 0; % Set = 1, if you want to save figures as encapsulated postscript files.

DO_CALC_XX = 1;                  % Set to 0, if XX is given
N_GRIDPOINTS = 64;               % gridpoints for inverse Fourier transform.  Needs to be a power of 2.
MOM_TOL =  0.0001;               % If PP has roots below TOL in absolute value,
                                 % then PP may be singular, and a safer but slower route to calculate
                                 % everything is taken.
HP_LAMBDA = 1600*(PERIOD/4)^4;   % The lambda parameter for the HP-Filter
N_LEADS_LAGS = 6;                % calculate autocorr. table up to N_LEADS_LAGS-1 leads and lags.

DO_HP_GRAPH = 0;    % Set to = 1 to see plots of the spectral densities.
DO_DISP1    = 1;    % Set to = 1 to see printout of the autocorrelation matrix. 
DO_DISP2    = 0;    % Set to = 1 to see printout of the variance-covariance matrix.
DO_DISP3    = 1;    % Set to = 1 to see printout of the vector of variances.

MOM_TXT_MARKER = N_GRIDPOINTS/24; 

DO_HP_FILTER= 1;   % Set to = 1, if you want moments and simulations with HP-filter.
                   % set to = 0, if you want moments and simulations without HP-filter.

[l_equ,m_states] = size(AA);
[l_equ,n_endog ] = size(CC);
[l_equ,k_exog  ] = size(DD);

DO_PLOTS   = 1;            % if impulse response plots should be made, = 0, if not.
% IMP_SELECT = 1:(m_states+n_endog+k_exog);
IMP_SELECT = [1:5,7];      %  a vector containing the indices of the variables to be plotted
HP_SELECT  = 1:(m_states+n_endog+k_exog); % Selecting the variables for the HP Filter calcs.
GNP_INDEX  = 3;           % Index of output among the variables selected for HP filter
goo = 2;

% Starting the calculations:

DO_IT;


 
       

