% RBC_capu_solution.m: RBC model with indivisible labor a-la Hansen (1985) and capacity utilization
 
%  Parameters:
%N_bar     = 1.0/3;  % Steady state employment is a third of total time endowment
%Z_bar     = 1;       % Normalization
alpha       = .36;    % Capital share
d1          = .025;   % Depreciation parameters
d2          = 1.6;
R_bar       = 1.01;   % One percent real interest per quarter
rho         = .95;   % autocorrelation of technology shock
sigma_eps = .712;    % Standard deviation of technology shock.  Units: Percent.
AA         = 2.86;   % parameter in the utility function

fix=alpha/(1-alpha);

% Steady state:
betta   = 1.0/R_bar;  
u_bar    = ((1/betta-1)/d1*(d2-1))^(1.0/d2);
YK_bar  = (d1*d2/alpha)*u_bar^d2; 
iK_bar  = d1*d2*u_bar^d2;
NY_bar = (YK_bar/u_bar)^fix;
C_bar  = (1/NY_bar)*(1-alpha)/AA;
IY_bar = iK_bar/YK_bar;
CY_bar = 1- IY_bar;
Y_bar  = C_bar/CY_bar;
I_bar  = IY_bar*Y_bar;
K_bar  = Y_bar/YK_bar;
N_bar  = NY_bar/Y_bar;

w1 = d1*u_bar^d2;
w2 = w1*d2;

VARNAMES = ['capital    ',
            'consumption',
            'output     ',
            'labor      ',
            'interest   ',
            'investment ',
            'utilization',
            'technology '];

 
% LOG-LINEARIZED CONDITIONS  
%
% 1) 0 = - I i(t) - C c(t) + Y y(t)
% 2) 0 = I i(t) - K k(t) + (1-w1) K k(t-1)+ w2*u(t)
% 3) 0 = alpha k(t-1) - y(t) + (1-alpha) n(t) + z(t)
% 4) 0 = -c(t) + y(t) - n(t)
% 5) 0 = - alpha Y/K k(t-1) + alpha Y/K y(t) - w2*u(t) - R r(t)
% 6) 0 = d2 u(t) - y(t) - k(t-1) 
% 7) 0 = E_t [ - c(t+1) + r(t+1) + c(t) ]
% 8) z(t+1) = psi z(t) + epsilon(t+1)
% 8 equations, 8 variables.
%
% Endogenous state variables "x(t)": k(t)
% Endogenous other variables "y(t)": c(t), y(t), n(t), r(t), i(t) u(t)
% Exogenous state variables  "z(t)": z(t).

% Find matrices for format
% 0 = AA x(t) + BB x(t-1) + CC y(t) + DD z(t)
% 0 = E_t [ FF x(t+1) + GG x(t) + HH x(t-1) + JJ y(t+1) + KK y(t) + LL z(t+1) + MM z(t)]
% z(t+1) = NN z(t) + epsilon(t+1) with E_t [ epsilon(t+1) ] = 0,

%  k(t):
AA = [ 0
       - K_bar
       0
       0
       0 
       0];

%  k(t-1):
BB = [ 0
       (1-w1)*K_bar
       alpha
       0
       - alpha * YK_bar 
       -1];

%        consumption  output      labor     interest  investment    utilization
CC = [    -C_bar,      Y_bar,      0,        0,        -I_bar         0    % Equ. 1)
          0,           0,          0,        0,        I_bar          w2   % Equ. 2)
          0,           -1,         1-alpha,    0,        0            0    % Equ. 3)      
          1,            1,          -1,       0,        0             0     % Equ. 4)
          0,           alpha*YK_bar, 0,        - R_bar,  0            -w2   % eqn 5)
          0,           -1           0        0           0            -d2]; % Equ. 6)

DD = [ 0
       0
       1
       0
       0 
       0];

FF = [ 0 ];

GG = [ 0 ];

HH = [ 0 ];

JJ = [ -1,  0,  0,  1,  0,  0];

KK = [ 1,   0,  0,  0,  0, 0 ];

LL = [ 0 ];

MM = [ 0 ];

NN = [rho];

Sigma = [ sigma_eps^2  ];

% Setting the options:
HORIZON    = 20; % how far out should the impulse responses be calculated
PERIOD     = 4;  % number of periods per year, i.e. 12 for monthly, 4 for quarterly

TXT_MARKER = 3; % Where to put the labels on the graph, a number between 1 and HORIZON/2.
DO_ENLARGE = 0; % =1, if you need large letter sizes for overheads, say.
PRINT_FIG  = 0; % Set = 1, if you want to sent figures directly to printer
SAVE_FIG   = 0; % Set = 1, if you want to save figures as encapsulated postscript files.

DO_CALC_XX = 1;         % Set to 0, if XX is given
N_GRIDPOINTS = 64;      % gridpoints for inverse Fourier transform.  Needs to be a power of 2.
MOM_TOL =  0.0001;      % If PP has roots below TOL in absolute value,
                        % then PP may be singular, and a safer but slower route to calculate
                        % everything is taken.
HP_LAMBDA = 1600*(PERIOD/4)^4;    % The lambda parameter for the HP-Filter
N_LEADS_LAGS = 6;  % calculate autocorr. table up to N_LEADS_LAGS-1 leads and lags.

DO_HP_GRAPH = 0;    % Set to = 1 to see plots of the spectral densities.
DO_DISP1    = 1;    % Set to = 1 to see printout of the autocorrelation matrix. 
DO_DISP2    = 0;    % Set to = 1 to see printout of the variance-covariance matrix.
DO_DISP3    = 1;    % Set to = 1 to see printout of the vector of variances.

MOM_TXT_MARKER = N_GRIDPOINTS/24; 

DO_HP_FILTER= 1;   % Set to = 1, if you want moments and simulations with HP-filter.
                      % set to = 0, if you want moments and simulations without HP-filter.

[l_equ,m_states] = size(AA);
[l_equ,n_endog ] = size(CC);
[l_equ,k_exog  ] = size(DD);

DO_PLOTS   = 1;  % if impulse response plots should be made, = 0, if not.
% IMP_SELECT = 1:(m_states+n_endog+k_exog);
IMP_SELECT = [1:5,7:8];
   %  a vector containing the indices of the variables to be plotted
HP_SELECT  = 1:(m_states+n_endog+k_exog); % Selecting the variables for the HP Filter calcs.
GNP_INDEX  = 3; % Index of output among the variables selected for HP filter
goo = 2;

% Starting the calculations:

do_it;


 
       

