% solution_check.m , February 15, 2009, version 1.3
% Solves a sticky price model of 3 equations in output gap, inflation, nominal rate
% with habit persistence in consumption and price indexation.
% model is:
% y(t) = gam/(1+gam) y(t-1)+ 1/(1+gam) E_{t} y_{t+1} + 1/ phi (i(t)-E_t(Delta p_{t+1})) + e1(t)
% Delta p(t) = omega/ (1+omega beta) Delta p(t-1) + beta/(1+omega beta) (Delta p(t+1) +
%      ((phi+vartheta)*(1-zeta_p beta)(1-zeta_p))/((1+omega beta)*zeta_p) y_t + e2(t)
% i_t       = phi_r i_(t-1)+ (1-phi_r)(phi_p Delta p_t +phi_x y(t)) + e3(t)

% checks if the simulated data satisfy the euler equation (in the form
% E_t[ beta* U(c,t+1)/U(c,t) i(t)/Delta p(t+1)]-1=0

clear all;
clc;
% Setting parameters:

% economic parameters:
bet = 0.99 ; % discount factor
phi = 2.0; % relative risk aversion coefficient
zeta_p = 0.68; % degree of price stickiness
vartheta = 3.0; % elasticity of labor supply
phi_r = 0.6; % interest rate smoothing parameter
phi_p = 1.5; % weight on inflation in interest rate rule 
phi_x = 1.1; % weight on output gap in interest rate rule
omega = 0.4; %price indexation parameter
gam = 0.7 ; % habit persistence parameter
zetaps = (1-zeta_p)*(1-bet*zeta_p)/(zeta_p);
% auxiliary parameters
rho1 = 0.65; 
rho2 = 0.65; 
sigma_e1 = 0.01; 
sigma_e2 = 0.01; 
sigma_e3 = 0.01;

VARNAMES = ['interest rate ',
            'inflation     ',
            'output gap    ',
            'lag inflation ',
            'lag gap       ',
            'IS shock      ',
            'AS shock      ',
            'Taylor shock  '];

 
% Endogenous state variables "x(t)": interest rate, inflation, gap
% Endogenous other variables "y(t)": lagged inflation and lagged gap
% Exogenous state variables  "z(t)": 3 equation shocks
% Switch to that notation.  Find matrices for format
% 0 = AA x(t) + BB x(t-1) + CC y(t) + DD z(t)
% 0 = E_t [ FF x(t+1) + GG x(t) + HH x(t-1) + JJ y(t+1) + KK y(t) + LL z(t+1) + MM z(t)]
% z(t+1) = NN z(t) + epsilon(t+1) with E_t [ epsilon(t+1) ] = 0,

% for i(t),  pi(t),   x(t):
AA = [ -1,    0,         0
        0,    0,         0
        0,    0,         0]; 

% for i(t-1), pi(t-1),   x(t-1):
BB = [ phi_r,   0,       0
          0,    1,       0
          0,    0,       1]; 

%Order:   lag-pi(t-1), lag-x(t-1)
CC = [    (1-phi_r)*phi_p, (1-phi_r)*phi_x
                       -1,      0
                        0,     -1];

%     e1(t)   e2(t) e3(t) 
DD = [  0,     0,    1
        0,    0,    0
        0,    0,    0]; 


% for i(t+1),  pi(t+1),            x(t+1):
FF = [  0,    1/phi,             1/(1+gam)
        0,    bet/(1+omega*bet),        0];


% for i(t),       pi(t),   x(t):
GG = [ -1/phi,    0,        -1
            0,    -1,     (phi+vartheta)*zetaps/(1+omega*bet)];
        
% for i(t-1),     pi(t-1),   x(t-1):

HH = [ 0,  0,  0
       0,  0,  0];
% for lag-pi(t+1), lag-x(t+1)

JJ = [ 0,  0
       0,  0];

   % for lag-pi(t), lag-x(t)

KK = [ 0,  gam/(1+gam)
    omega/(1+bet*omega),0];

%     e1(t+1)   e2(t+1) e3(t+1) 
LL = [ 0,  0,  0
       0,  0,  0];

%     e1(t)   e2(t) e3(t) 

MM = [ 1,  0,  0
       0,  1,  0];

NN = zeros(3,3);
NN(1,1) = rho1;
NN(2,2) = rho2;
NN(3,3) = 0;

Sigma = [ sigma_e1^2,          0,        0
                   0, sigma_e2^2,        0
                   0,          0, sigma_e3^2];

               
% solve the model
               
solving;    
 
 
% simulating data out of the model 
yy= zeros(8,600);
yy0=zeros(8,1); % 
sig=zeros(3,3);
sig(1,1)= 0.001;  % these need to be appropriately calibrated
sig(2,2)= 0.001;
sig(3,3)= 0.001;

[m_states,k_exog] = size(QQ);
[n_endog,k_exog]  = size(SS);
 
II_lag = [ PP, zeros(m_states,n_endog),zeros(m_states,k_exog)
           RR, zeros(n_endog, n_endog),zeros(n_endog, k_exog)
           zeros(k_exog,(m_states+n_endog)), NN                ];
              
II_contemp = eye(m_states+n_endog+k_exog) + ...
        [ zeros(m_states,(m_states+n_endog)), -QQ
          zeros(n_endog, (m_states+n_endog)), -SS
          zeros(k_exog,  (m_states+n_endog)), zeros(k_exog,k_exog) ];
       

 ee=zeros(8,600);
 ee(6:8,:)=sig*randn(3,600);
 for tt=1:600;
    yy(:,tt) = II_lag *yy0 +II_contemp*ee(:,tt);
    yy0=yy(:,tt);
end 


% yy(1) is interest rate, yy(2) is inflation, yy(3) is the output gap
% Euler equation is
% u(1t)= bet*((y(t+1) - gam* y(t))/(y(t) - gam* y(t-1)))^phi * (i(t)/Delta p(t+1)) -1
 
for t=2:599
uu1(t)= real((bet*(yy(1,t)/yy(2,t+1))*((yy(3,t+1)- gam*yy(3,t))/(yy(3,t)- gam*yy(3,t-1)))^(phi))-1); 
end

%graphing errors
figure(1)
plot((uu1(101:400)),'-');
title('IS error');
 
aa= mean(uu1);
bb=std(uu1);
disp('mean and std of approximation error')
[aa bb]

%yy1=zeros(3,500);
%yy1(:)=yy(1:3,101:600)';

%saving simulated data
%diary simulation3c.dat
%yy1'
%diary off
